from contextlib import asynccontextmanager
from enum import Enum
from typing import Optional

class BrowserMode(str, Enum):
    PLAYWRIGHT = 'PLAYWRIGHT'
    NONEBOT_HTMLRENDER = 'NONEBOT_HTMLRENDER'

class BrowserAdapter:
    """浏览器适配器"""
    def __init__(
            self,
            mode: BrowserMode,
    ):
        self.mode = mode
        self.browser = None

    @asynccontextmanager
    async def get_new_page(
            self,
            device_scale_factor: float = 1.0,
            viewport: Optional[dict] = None,
    ):
        kwargs = {}
        if device_scale_factor and device_scale_factor != 1.0:
            kwargs['device_scale_factor'] = device_scale_factor
        if viewport:
            kwargs['viewport'] = viewport
        
        page = None
        try:
            if self.mode == BrowserMode.NONEBOT_HTMLRENDER:
                from nonebot_plugin_htmlrender import get_new_page as htmlrender_get_page
                async with htmlrender_get_page(**kwargs) as p:
                    yield p
            elif self.mode == BrowserMode.PLAYWRIGHT:
                from .browser_adapter_playwright import get_new_page as playwright_get_page
                async with playwright_get_page(**kwargs) as p:
                    yield p
            else:
                raise ValueError(f"Unsupported browser mode: {self.mode}")
        finally:
            if page and not page.is_closed():
                await page.close()
    



from pydantic import BaseModel, Field
from nonebot import get_plugin_config


class Config(BaseModel):
    """Plugin Config Here"""

    bili_helper_tmp_dir: str = (Field("./tmp/bili_helper", doc="B站解析临时文件存储位置"))
    bili_helper_cookie_path: str = (Field("./data/bili_helper_cookie.json", doc="B站Cookie存储位置"))
    analysis_whitelist: list[str] = (Field(default_factory=list, doc="B站解析用户白名单"))
    analysis_group_whitelist: list[str] = (Field(default_factory=list, doc="B站解析群组白名单"))
    analysis_blacklist: list[str] = (Field(default_factory=list, doc="B站解析用户黑名单"))
    analysis_group_blacklist: list[str] = (Field(default_factory=list, doc="B站解析群组黑名单"))
    analysis_group_strategies: dict[str, list[str]] = (Field({}, doc="B站解析链接群组策略"))

config = get_plugin_config(Config)



import asyncio
import os
import sys

DIRNAME = os.path.dirname(os.path.abspath(__file__))
ROOT_DIR = os.path.dirname(DIRNAME)
sys.path.append(ROOT_DIR)

from modules.browser_adapter import BrowserAdapter, BrowserMode

TMP_DIR = os.path.join(ROOT_DIR, 'tmp')

os.makedirs(TMP_DIR, exist_ok=True)

async def test_browser_adapter():
    url = 'file://' + os.path.join(ROOT_DIR, 'resources/render-v2/bilibili/comment.html?mock=1')
    adapter = BrowserAdapter(mode=BrowserMode.PLAYWRIGHT)
    async with adapter.get_new_page() as page:
        print('- Browser launched')
        await page.goto(url)
        print('- Page loaded')
        await page.wait_for_selector('.comment-container', timeout=15000)
        await asyncio.sleep(.5)  # 等待额外的 JS 执行
        screenshot = await page.screenshot(full_page=True, type='jpeg', quality=80)
        screenshot_path = os.path.join(TMP_DIR, 'test-browser-adapter.jpg')
        with open(screenshot_path, 'wb') as f:
            f.write(screenshot)
        print(f'- Screenshot saved to {screenshot_path}')

def main():
    import asyncio
    asyncio.run(test_browser_adapter())

if __name__ == '__main__':
    main()



from typing import Tuple
import urllib.parse
import time
import requests
from functools import reduce
from hashlib import md5

from .api_base import ApiEncoder

class WbiEncoder(ApiEncoder):
    'B 站 WBI 签名算法'

    mixin_key_enc_tab = [
        46, 47, 18, 2, 53, 8, 23, 32, 15, 50, 10, 31, 58, 3, 45, 35, 27, 43, 5, 49,
        33, 9, 42, 19, 29, 28, 14, 39, 12, 38, 41, 13, 37, 48, 7, 16, 24, 55, 40,
        61, 26, 17, 0, 1, 60, 51, 30, 4, 22, 25, 54, 21, 56, 59, 6, 63, 57, 62, 11,
        36, 20, 34, 44, 52
    ]

    def __init__(self, ua: str, refer: str = '', cookie: str = ''):
        self.ua = ua
        self.refer = refer
        self.cookie = cookie
        self.keys = self.get_wbi_keys()

    def set_cookie(self, cookie: str):
        self.cookie = cookie
        self.keys = self.get_wbi_keys()

    def encode(self, url: str, params: dict) -> Tuple[str, dict]:
        new_params = self.enc_wbi(params)
        return url, new_params

    @staticmethod
    def get_mixin_key(orig: str):
        '对 imgKey 和 subKey 进行字符顺序打乱编码'
        return reduce(lambda s, i: s + orig[i], WbiEncoder.mixin_key_enc_tab, '')[:32]

    def enc_wbi(self, params: dict):
        '为请求参数进行 wbi 签名'
        img_key, sub_key = self.keys
        mixin_key = WbiEncoder.get_mixin_key(img_key + sub_key)
        curr_time = round(time.time())
        params['wts'] = curr_time                                   # 添加 wts 字段
        params = dict(sorted(params.items()))                       # 按照 key 重排参数
        # 过滤 value 中的 "!'()*" 字符
        params = {
            k : ''.join(filter(lambda chr: chr not in "!'()*", str(v)))
            for k, v 
            in params.items()
        }
        query = urllib.parse.urlencode(params)                      # 序列化参数
        wbi_sign = md5((query + mixin_key).encode()).hexdigest()    # 计算 w_rid
        params['w_rid'] = wbi_sign
        return params

    def get_wbi_keys(self) -> Tuple[str, str]:
        '获取最新的 img_key 和 sub_key'
        headers = {
            'User-Agent': self.ua,
            'Referer': self.refer,
            'Cookie': self.cookie,
        }
        resp = requests.get('https://api.bilibili.com/x/web-interface/nav', headers=headers)
        resp.raise_for_status()
        json_content = resp.json()
        img_url: str = json_content['data']['wbi_img']['img_url']
        sub_url: str = json_content['data']['wbi_img']['sub_url']
        img_key = img_url.rsplit('/', 1)[1].split('.')[0]
        sub_key = sub_url.rsplit('/', 1)[1].split('.')[0]
        return img_key, sub_key



import asyncio
from contextlib import asynccontextmanager
from playwright.async_api import async_playwright, Browser, Playwright
import sys
from typing import Optional, List, Callable

chrome_path = {
    'win32': 'C:\\Program Files\\Google\\Chrome\\Application\\chrome.exe',
    'darwin': '/Applications/Google Chrome.app/Contents/MacOS/Google Chrome',
    'linux': '/usr/bin/google-chrome',
}[sys.platform]

use_headless = sys.platform in ['darwin', 'linux']
use_chrome = sys.platform in ['win32', 'darwin']

print(f'Browser Adapter Playwright: use_headless={use_headless}, use_chrome={use_chrome}, chrome_path={chrome_path}')

class BrowserManager:
    def __init__(self):
        self._runtime_instance: Optional[Browser] = None
        self._p_holding: Optional[Playwright] = None
        self._instance_creating: bool = False
        self._instance_waiting: List[Callable] = []

    def close(self):
        p = self._p_holding
        browser = self._runtime_instance
        self._p_holding = None
        self._runtime_instance = None
        if p:
            asyncio.create_task(p.stop())
        if browser:
            asyncio.create_task(browser.close())

    async def _create_instance(self):
        if self._instance_creating:
            raise RuntimeError("Instance is creating")
        self._instance_creating = True
        ap = async_playwright()
        p = await ap.start()
        self._p_holding = p
        browser = await p.chromium.launch(
            headless=use_headless,
            executable_path=chrome_path if use_chrome else None,
        )
        self._runtime_instance = browser
        self._instance_creating = False
        self._flush_waitings()
        return browser
    
    def _flush_waitings(self):
        if len(self._instance_waiting) < 1:
            return
        for task in self._instance_waiting:
            task(self._runtime_instance)
        self._instance_waiting = []

    async def get_instance(self):
        if not self._runtime_instance:
            if not self._instance_creating:
                await self._create_instance()
                return await self.get_instance()
            # 创建一个 Future 来等待实例创建完成
            future = asyncio.Future()
            self._instance_waiting.append(future.set_result)
            return await future
        return self._runtime_instance

browser_manager = BrowserManager()

@asynccontextmanager
async def get_new_page(
        device_scale_factor: float = 1.0,
        viewport: Optional[dict] = None,
):
    print('Getting new page...')
    browser = await browser_manager.get_instance()
    print('Got browser instance.')
    context = await browser.new_context(
        device_scale_factor = device_scale_factor,
        viewport = viewport
    )
    print('Created new context.')
    page = await context.new_page()
    try:
        print('Created new page.')
        yield page
        print('Yielded page.')
    finally:
        print('Closing page and context...')
        await page.close()
        await context.close()
        print('Closed page and context.')

